@@ -1,11 +1,11 @@
 /*
  *  OpenVPN -- An application to securely tunnel IP networks
- *             over a single UDP port, with support for TLS-based
+ *             over a single UDP port, with support for SSL/TLS-based
  *             session authentication and key exchange,
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,16 +39,20 @@ struct buffer
 
 #define BPTR(buf)  ((buf)->data + (buf)->offset)
 #define BEND(buf)  (BPTR(buf) + (buf)->len)
-#define BLAST(buf) ((buf)->len ? BPTR(buf) + (buf)->len - 1 : NULL)
+#define BLAST(buf) (((buf)->data && (buf)->len) ? (BPTR(buf) + (buf)->len - 1) : NULL)
 #define BLEN(buf)  ((buf)->len)
 #define BDEF(buf)  ((buf)->data != NULL)
+#define BSTR(buf)  ((char *)BPTR(buf))
+#define BCAP(buf)  (buf_forward_capacity (buf))
 
 struct buffer alloc_buf (size_t size);
 struct buffer clone_buf (const struct buffer* buf);
 struct buffer alloc_buf_gc (size_t size);	/* allocate buffer with garbage collection */
-struct buffer clear_buf ();
+struct buffer clear_buf (void);
 void free_buf (struct buffer *buf);
 
+/* inline functions */
+
 static inline bool
 buf_init (struct buffer *buf, int offset)
 {
@@ -59,6 +63,12 @@ buf_init (struct buffer *buf, int offset)
   return true;
 }
 
+static inline bool
+buf_defined (struct buffer *buf)
+{
+  return buf->data != NULL;
+}
+
 static inline void
 buf_clear (struct buffer *buf)
 {
@@ -86,17 +96,56 @@ buf_set_read (struct buffer *buf, uint8_t *data, int size)
 
 /* Like strncpy but makes sure dest is always null terminated */
 static inline void
-strncpynt (char *dest, const char *src, int maxlen)
+strncpynt (char *dest, const char *src, size_t maxlen)
 {
   strncpy (dest, src, maxlen);
   if (maxlen > 0)
     dest[maxlen - 1] = 0;
 }
 
+/* return true if string contains at least one numerical digit */
+static inline bool
+has_digit (const char* src)
+{
+  char c;
+  while ((c = *src++))
+    {
+      if (isdigit(c))
+	return true;
+    }
+  return false;
+}
+
 /*
  * printf append to a buffer with overflow check
  */
-void buf_printf (struct buffer *buf, char *format, ...);
+void buf_printf (struct buffer *buf, const char *format, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 2, 3)))
+#endif
+    ;
+
+/*
+ * Like snprintf but guarantees null termination for size > 0
+ */
+int openvpn_snprintf(char *str, size_t size, const char *format, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 3, 4)))
+#endif
+    ;
+
+/*
+ * remove trailing characters
+ */
+
+void buf_rmtail (struct buffer *buf, uint8_t remove);
+void chomp (char *str);
+
+/*
+ * Write string in buf to file descriptor fd.
+ * NOTE: requires that string be null terminated.
+ */
+void buf_write_string_file (const struct buffer *buf, const char *filename, int fd);
 
 /*
  * write a string to the end of a buffer that was
@@ -104,12 +153,18 @@ void buf_printf (struct buffer *buf, char *format, ...);
  */
 void buf_catrunc (struct buffer *buf, const char *str);
 
+/*
+ * convert a multi-line output to one line
+ */
+void convert_to_one_line (struct buffer *buf);
+
+
 /*
  * Hex dump -- Output a binary buffer to a hex string and return it.
  */
 char *
 format_hex_ex (const uint8_t *data, int size, int maxoutput,
-	       int space_break, char* separator);
+	       int space_break, const char* separator);
 
 static inline char *
 format_hex (const uint8_t *data, int size, int maxoutput)
@@ -129,7 +184,7 @@ struct buffer buf_sub (struct buffer *buf, int size, bool prepend);
 static inline bool
 buf_safe (struct buffer *buf, int len)
 {
-  return buf->offset + buf->len + len <= buf->capacity;
+  return len >= 0 && buf->offset + buf->len + len <= buf->capacity;
 }
 
 static inline int
@@ -164,7 +219,7 @@ buf_reverse_capacity (struct buffer *buf)
 static inline uint8_t *
 buf_prepend (struct buffer *buf, int size)
 {
-  if (size > buf->offset)
+  if (size < 0 || size > buf->offset)
     return NULL;
   buf->offset -= size;
   buf->len += size;
@@ -174,7 +229,7 @@ buf_prepend (struct buffer *buf, int size)
 static inline bool
 buf_advance (struct buffer *buf, int size)
 {
-  if (buf->len < size)
+  if (size < 0 || buf->len < size)
     return false;
   buf->offset += size;
   buf->len -= size;
@@ -207,7 +262,7 @@ static inline uint8_t *
 buf_read_alloc (struct buffer *buf, int size)
 {
   uint8_t *ret;
-  if (buf->len < size)
+  if (size < 0 || buf->len < size)
     return NULL;
   ret = BPTR (buf);
   buf->offset += size;
@@ -235,12 +290,83 @@ buf_write_prepend (struct buffer *dest, const void *src, int size)
   return true;
 }
 
+static inline bool
+buf_write_u8 (struct buffer *dest, int data)
+{
+  uint8_t u8 = (uint8_t) data;
+  return buf_write (dest, &u8, sizeof (uint8_t));
+}
+
+static inline bool
+buf_write_u16 (struct buffer *dest, int data)
+{
+  uint16_t u16 = htons ((uint16_t) data);
+  return buf_write (dest, &u16, sizeof (uint16_t));
+}
+
+static inline bool
+buf_write_u32 (struct buffer *dest, int data)
+{
+  uint32_t u32 = htonl ((uint32_t) data);
+  return buf_write (dest, &u32, sizeof (uint32_t));
+}
+
 static inline bool
 buf_copy (struct buffer *dest, const struct buffer *src)
 {
   return buf_write (dest, BPTR (src), BLEN (src));
 }
 
+static inline bool
+buf_copy_n (struct buffer *dest, struct buffer *src, int n)
+{
+  uint8_t *cp = buf_read_alloc (src, n);
+  if (!cp)
+    return false;
+  return buf_write (dest, cp, n);
+}
+
+static inline bool
+buf_copy_range (struct buffer *dest,
+		int dest_index,
+		const struct buffer *src,
+		int src_index,
+		int src_len)
+{
+  if (src_index < 0
+      || src_len < 0
+      || src_index + src_len > src->len
+      || dest_index < 0
+      || dest->offset + dest_index + src_len > dest->capacity)
+    return false;
+  memcpy (dest->data + dest->offset + dest_index, src->data + src->offset + src_index, src_len);
+  if (dest_index + src_len > dest->len)
+    dest->len = dest_index + src_len;
+  return true;
+}
+
+/* truncate src to len, copy excess data beyond len to dest */
+static inline bool
+buf_copy_excess (struct buffer *dest,
+		 struct buffer *src,
+		 int len)
+{
+  if (len < 0)
+    return false;
+  if (src->len > len)
+    {
+      struct buffer b = *src;
+      src->len = len;
+      if (!buf_advance (&b, len))
+	return false;
+      return buf_copy (dest, &b);
+    }
+  else
+    {
+      return true;
+    }
+}
+
 static inline bool
 buf_read (struct buffer *src, void *dest, int size)
 {
@@ -251,6 +377,44 @@ buf_read (struct buffer *src, void *dest, int size)
   return true;
 }
 
+static inline int
+buf_read_u8 (struct buffer *buf)
+{
+  int ret;
+  if (BLEN (buf) < 1)
+    return -1;
+  ret = *BPTR(buf);
+  buf_advance (buf, 1);
+  return ret;
+}
+
+static inline int
+buf_read_u16 (struct buffer *buf)
+{
+  uint16_t ret;
+  if (!buf_read (buf, &ret, sizeof (uint16_t)))
+    return -1;
+  return ntohs (ret);
+}
+
+static inline uint32_t
+buf_read_u32 (struct buffer *buf, bool *good)
+{
+  uint32_t ret;
+  if (!buf_read (buf, &ret, sizeof (uint32_t)))
+    {
+      if (good)
+	*good = false;
+      return 0;
+    }
+  else
+    {
+      if (good)
+	*good = true;
+      return ntohl (ret);
+    }
+}
+
 static inline bool
 buf_string_match (struct buffer *src, const void *match, int size)
 {
@@ -259,6 +423,24 @@ buf_string_match (struct buffer *src, const void *match, int size)
   return memcmp (BPTR (src), match, size) == 0;
 }
 
+static inline bool
+buf_string_match_head (struct buffer *src, const void *match, int size)
+{
+  if (size < 0 || size > src->len)
+    return false;
+  return memcmp (BPTR (src), match, size) == 0;
+}
+
+/*
+ * Bitwise operations
+ */
+static inline void
+xor (uint8_t *dest, const uint8_t *src, int len)
+{
+  while (len-- > 0)
+    *dest++ ^= *src++;
+}
+
 /*
  * Very basic garbage collection, mostly for routines that return
  * char ptrs to malloced strings.
@@ -277,24 +459,43 @@ struct gc_thread
   struct gc_entry *gc_stack;
 };
 
-extern struct gc_thread _gc_thread[N_THREADS];
+extern struct gc_thread x_gc_thread[N_THREADS];
 
 void *gc_malloc (size_t size);
-void gc_collect (int level);
 
-void _gc_free (void *p);
+static inline void
+x_gc_free (void *p) {
+  free (p);
+}
+
+static inline void
+gc_collect (int level)
+{
+  struct gc_entry *e;
+  struct gc_thread* thread = &x_gc_thread[thread_number()];
+
+  while ((e = thread->gc_stack))
+    {
+      if (e->level < level)
+	break;
+      /*printf("GC FREE " ptr_format " lev=%d\n", e, e->level); */
+      --thread->gc_count;
+      thread->gc_stack = e->back;
+      x_gc_free (e);
+    }
+}
 
 static inline int
-gc_new_level ()
+gc_new_level (void)
 {
-  struct gc_thread* thread = &_gc_thread[thread_number()];
+  struct gc_thread* thread = &x_gc_thread[thread_number()];
   return ++thread->gc_level;
 }
 
 static inline void
 gc_free_level (int level)
 {
-  struct gc_thread* thread = &_gc_thread[thread_number()];
+  struct gc_thread* thread = &x_gc_thread[thread_number()];
 
   gc_collect (level);
   thread->gc_level = level - 1;
