@@ -1,11 +1,11 @@
 /*
  *  OpenVPN -- An application to securely tunnel IP networks
- *             over a single UDP port, with support for TLS-based
+ *             over a single UDP port, with support for SSL/TLS-based
  *             session authentication and key exchange,
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,12 +28,17 @@
  * network outages when the --gremlin option is enabled.
  */
 
+#ifdef WIN32
+#include "config-win32.h"
+#else
 #include "config.h"
+#endif
 
 #include "syshead.h"
 
 #include "error.h"
 #include "common.h"
+#include "misc.h"
 #include "gremlin.h"
 
 #include "memdbg.h"
@@ -79,7 +84,7 @@
  * Return true with probability 1/n
  */
 static bool flip(int n) {
-  return (random() % n) == 0;
+  return (get_random() % n) == 0;
 }
 
 /*
@@ -89,7 +94,7 @@ static bool flip(int n) {
 static int roll(int low, int high) {
   int ret;
   ASSERT (low <= high);
-  ret = low + (random() % (high - low + 1));
+  ret = low + (get_random() % (high - low + 1));
   ASSERT (ret >= low && ret <= high);
   return ret;
 }
@@ -102,26 +107,24 @@ static time_t next;
  * Return false if we should drop a packet.
  */
 bool
-ask_gremlin()
+ask_gremlin(void)
 {
-  struct timeval tv;
-  ASSERT (!gettimeofday (&tv, NULL));
+  const time_t current = time (NULL);
 
   if (!initialized)
     {
-      srandom (tv.tv_usec);
       initialized = true;
 #ifdef UP_DOWN_ENABLE
       up = false;
 #else
       up = true;
 #endif
-      next = tv.tv_sec;
+      next = current;
     }
 
 #ifdef UP_DOWN_ENABLE
 /* change up/down state? */
-  if (tv.tv_sec >= next)
+  if (current >= next)
     {
       int delta;
       if (up)
@@ -139,14 +142,14 @@ ask_gremlin()
 	   "GREMLIN: CONNECTION GOING %s FOR %d SECONDS",
 	   (up ? "UP" : "DOWN"),
 	   delta);
-      next = tv.tv_sec + delta;
+      next = current + delta;
     }
 #endif
 
 #ifdef DROP_ENABLE
   if (up && flip (DROP_FREQ))
     {
-      msg (D_GREMLIN_VERBOSE, "GREMLIN: RANDOM PACKET DROP");
+      msg (D_GREMLIN_VERBOSE, "GREMLIN: Random packet drop");
       return false;
     }
 #endif
@@ -188,7 +191,7 @@ void corrupt_gremlin(struct buffer* buf) {
 		buf->len -= roll (0, buf->len - 1);
 		break;
 	      }
-	      msg (D_GREMLIN_VERBOSE, "GREMLIN: PACKET CORRUPTION, method=%d", method);
+	      msg (D_GREMLIN_VERBOSE, "GREMLIN: Packet Corruption, method=%d", method);
 	    }
 	  else
 	    break;
