@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2003 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@
 #include "io.h"
 #include "fragment.h"
 #include "proxy.h"
+#include "socks.h"
 #include "openvpn.h"
 #include "win32.h"
 
@@ -341,7 +342,7 @@ do_open_tun (const struct options *options,
       run_script (options->up_script,
 		  tuntap->actual,
 		  TUN_MTU_SIZE (frame),
-		  MAX_RW_SIZE_LINK (frame),
+		  EXPANDED_SIZE (frame),
 		  print_in_addr_t (tuntap->local, true),
 		  print_in_addr_t (tuntap->remote_netmask, true),
 		  "init",
@@ -360,6 +361,14 @@ do_open_tun (const struct options *options,
 			       frame,
 			       tuntap->post_open_mtu,
 			       SET_MTU_TUN | SET_MTU_UPPER_BOUND);
+
+      /*
+       * On Windows, it is usually wrong if --tun-mtu != 1500.
+       */
+#ifdef WIN32
+      if (TUN_MTU_SIZE (frame) != 1500)
+	msg (M_WARN, "WARNING: in general you should use '--tun-mtu 1500 --mssfix 1400' on both sides of the connection if at least one side is running Windows, unless you have explicitly modified the TAP-Win32 driver properties");
+#endif
       ret = true;
     }
   else
@@ -371,7 +380,7 @@ do_open_tun (const struct options *options,
 	run_script (options->up_script,
 		    tuntap->actual,
 		    TUN_MTU_SIZE (frame),
-		    MAX_RW_SIZE_LINK (frame),
+		    EXPANDED_SIZE (frame),
 		    print_in_addr_t (tuntap->local, true),
 		    print_in_addr_t (tuntap->remote_netmask, true),
 		    "restart",
@@ -386,19 +395,19 @@ do_open_tun (const struct options *options,
  * TCP race.
  */
 static void
-socket_restart_pause (int proto, bool http_proxy)
+socket_restart_pause (int proto, bool http_proxy, bool socks_proxy)
 {
   int sec = 0;
   switch (proto)
     {
     case PROTO_UDPv4:
-      sec = 0;
+      sec = socks_proxy ? 3 : 0;
       break;
     case PROTO_TCPv4_SERVER:
       sec = 1;
       break;
     case PROTO_TCPv4_CLIENT:
-      sec = http_proxy ? 10 : 3;
+      sec = (http_proxy || socks_proxy) ? 10 : 3;
       break;
     }
   if (sec)
@@ -587,6 +596,7 @@ openvpn (const struct options *options,
 	 struct packet_id_persist *pid_persist,
 	 struct route_list *route_list,
 	 struct http_proxy_info *http_proxy,
+	 struct socks_proxy_info *socks_proxy,
 	 bool first_time)
 {
   /*
@@ -787,7 +797,8 @@ openvpn (const struct options *options,
 #endif /* HAVE_SIGNAL_H */
 
   if (!first_time)
-    socket_restart_pause (options->proto, options->http_proxy_server != NULL);
+    socket_restart_pause (options->proto, options->http_proxy_server != NULL,
+			  options->socks_proxy_server != NULL);
 
   wait_init (&event_wait);
   link_socket_reset (&link_socket);
@@ -1039,7 +1050,7 @@ openvpn (const struct options *options,
 #else /* USE_CRYPTO */
 
   msg (M_WARN,
-       "******* WARNING *******: OpenVPN built without OpenSSL -- encryption and authentication features disabled -- all data will be tunnelled as cleartext");
+       "******* WARNING *******: " PACKAGE_NAME " built without OpenSSL -- encryption and authentication features disabled -- all data will be tunnelled as cleartext");
 
 #endif /* USE_CRYPTO */
 
@@ -1057,6 +1068,12 @@ openvpn (const struct options *options,
     }
 #endif
 
+  /*
+   * Adjust frame size for UDP Socks support.
+   */
+  if (options->socks_proxy_server)
+    socks_adjust_frame_parameters (&frame, options->proto);
+
   /*
    * Adjust frame size based on the --tun-mtu-extra parameter.
    */
@@ -1090,7 +1107,7 @@ openvpn (const struct options *options,
   if (tls_multi)
     {
       tls_multi_init_finalize (tls_multi, &frame);
-      ASSERT (MAX_RW_SIZE_LINK (&tls_multi->opt.frame) <= MAX_RW_SIZE_LINK (&frame));
+      ASSERT (EXPANDED_SIZE (&tls_multi->opt.frame) <= EXPANDED_SIZE (&frame));
       frame_print (&tls_multi->opt.frame, D_MTU_INFO, "Control Channel MTU parms");
     }
 #endif
@@ -1161,6 +1178,18 @@ openvpn (const struct options *options,
 #endif
     }
 
+#ifdef FRAGMENT_ENABLE
+
+  if (options->fragment && options->mtu_test)
+    msg (M_WARN, "WARNING: using --fragment and --mtu-test together may produce an inaccurate MTU test result");
+
+  if ((options->mssfix || options->fragment) && TUN_MTU_SIZE (&frame_fragment) != ETHERNET_MTU)
+     msg (M_WARN, "WARNING: normally if you use --mssfix and/or --fragment, you should also set --tun-mtu %d (currently it is %d)",
+	  ETHERNET_MTU,
+	  TUN_MTU_SIZE (&frame_fragment));
+
+#endif
+
   /* bind the TCP/UDP socket */
 
   link_socket_init_phase1 (&link_socket,
@@ -1168,12 +1197,14 @@ openvpn (const struct options *options,
 			   options->local_port, options->remote_port,
 			   options->proto,
 			   http_proxy->defined ? http_proxy : NULL,
+			   socks_proxy->defined ? socks_proxy : NULL,
 			   options->bind_local,
 			   options->remote_float,
 			   options->inetd,
 			   link_socket_addr,
 			   options->ipchange,
 			   options->resolve_retry_seconds,
+			   options->connect_retry_seconds,
 			   options->mtu_discover_type);
 
   /* initialize tun/tap device object */
@@ -1186,7 +1217,7 @@ openvpn (const struct options *options,
 	    addr_host (&link_socket.lsa->local),
 	    addr_host (&link_socket.lsa->remote),
 	    &frame,
-	    options->tuntap_flags);
+	    &options->tuntap_options);
 
   /* open tun/tap device, ifconfig, run up script, etc. */
   
@@ -1279,6 +1310,9 @@ openvpn (const struct options *options,
       if (options->mlock && did_we_daemonize)
 	do_mlockall (true); /* call again in case we daemonized */
 
+      /* should we change scheduling priority? */
+      set_nice (options->nice);
+
       /* set user and/or group that we want to setuid/setgid to */
       set_group (&group_state);
       set_user (&user_state);
@@ -1309,10 +1343,6 @@ openvpn (const struct options *options,
     }
 #endif
 
-  /* change scheduling priority if requested */
-  if (first_time)
-    set_nice (options->nice);
-
   /*
    * MAIN EVENT LOOP
    *
@@ -1382,6 +1412,12 @@ openvpn (const struct options *options,
       timeval.tv_usec = 0;
       tv = &timeval;
 
+#if defined(WIN32) && defined(TAP_WIN32_DEBUG)
+      timeval.tv_sec = 1;
+      if (check_debug_level (D_TAP_WIN32_DEBUG))
+	tun_show_debug (tuntap);
+#endif
+
 #ifdef USE_CRYPTO
       /* flush current packet-id to file once per 60
 	 seconds if --replay-persist was specified */
@@ -1420,20 +1456,22 @@ openvpn (const struct options *options,
 	      timeval.tv_sec = wakeup;
 	      timeval.tv_usec = 0;
 	    }
-
-	  if (link_socket_connection_oriented (&link_socket) && tls_multi->n_errors)
-	    {
-	      /* TLS errors are fatal in TCP mode */
-	      signal_received = SIGUSR1;
-	      msg (D_STREAM_ERRORS, "Fatal decryption error, restarting");
-	      signal_text = "tls-error";
-	      break;
-	    }
 	}
 #endif
 
       current = time (NULL);
 
+#if defined(USE_CRYPTO) && defined(USE_SSL)
+      if (tls_multi && link_socket_connection_oriented (&link_socket) && tls_multi->n_errors)
+	{
+	  /* TLS errors are fatal in TCP mode */
+	  signal_received = SIGUSR1;
+	  msg (D_STREAM_ERRORS, "Fatal decryption error, restarting");
+	  signal_text = "tls-error";
+	  break;
+	}
+#endif
+
       /*
        * Things that need to happen immediately after connection initiation should go here.
        */
@@ -1531,7 +1569,7 @@ openvpn (const struct options *options,
 		     * No OCC_REPLY from peer after repeated attempts.
 		     * Give up.
 		     */
-		    msg (D_SHOW_OCC, "NOTE: failed to obtain options consistency info from peer -- this could occur if the remote peer is running a version of OpenVPN before 1.5-beta8 or if there is a network connectivity problem, and will not necessarily prevent OpenVPN from running (%u bytes received from peer, %u bytes authenticated data channel traffic) -- you can disable the options consistency check with --disable-occ.", (unsigned int) link_read_bytes, (unsigned int) link_read_bytes_auth);
+		    msg (D_SHOW_OCC, "NOTE: failed to obtain options consistency info from peer -- this could occur if the remote peer is running a version of " PACKAGE_NAME " before 1.5-beta8 or if there is a network connectivity problem, and will not necessarily prevent " PACKAGE_NAME " from running (%u bytes received from peer, %u bytes authenticated data channel traffic) -- you can disable the options consistency check with --disable-occ.", (unsigned int) link_read_bytes, (unsigned int) link_read_bytes_auth);
 		  event_timeout_clear (&occ_interval);
 		}
 	      else
@@ -1567,7 +1605,7 @@ openvpn (const struct options *options,
 		  if (entry->op >= 0)
 		    {
 		      occ_op = entry->op;
-		      occ_mtu_load_size = MAX_RW_SIZE_LINK (&frame) + entry->delta;
+		      occ_mtu_load_size = EXPANDED_SIZE (&frame) + entry->delta;
 		    }
 		  else
 		    {
@@ -1582,12 +1620,16 @@ openvpn (const struct options *options,
       /*
        * Should we send an OCC message?
        */
-      if (occ_op >= 0 && !to_link.len)
+      if (occ_op >= 0 && !to_link.len
+#ifdef FRAGMENT_ENABLE
+	  && (!fragment || !fragment_outgoing_defined (fragment))
+#endif
+	  )
 	{
 	  bool doit = false;
 
 	  buf = aux_buf;
-	  ASSERT (buf_init (&buf, EXTRA_FRAME (&frame)));
+	  ASSERT (buf_init (&buf, FRAME_HEADROOM (&frame)));
 	  ASSERT (buf_safe (&buf, MAX_RW_SIZE_TUN (&frame)));
 	  ASSERT (buf_write (&buf, occ_magic, sizeof (occ_magic)));
 
@@ -1650,7 +1692,7 @@ openvpn (const struct options *options,
 				       - sizeof (occ_magic)
 				       - sizeof (uint8_t)
 				       - EXTRA_FRAME (&frame),
-				       MAX_RW_SIZE_TUN (&frame)); 
+				       EXPANDED_SIZE (&frame)); 
 		while (need_to_add > 0)
 		  {
 		    /*
@@ -1712,7 +1754,7 @@ openvpn (const struct options *options,
 	      if (event_timeout_trigger (&ping_send_interval, current, &timeval))
 		{
 		  buf = aux_buf;
-		  ASSERT (buf_init (&buf, EXTRA_FRAME (&frame)));
+		  ASSERT (buf_init (&buf, FRAME_HEADROOM (&frame)));
 		  ASSERT (buf_safe (&buf, MAX_RW_SIZE_TUN (&frame)));
 		  ASSERT (buf_write (&buf, ping_string, sizeof (ping_string)));
 
@@ -1845,7 +1887,7 @@ openvpn (const struct options *options,
 	{
 	  if (signal_received == SIGUSR2)
 	    {
-	      msg (M_INFO, "Current OpenVPN Statistics:");
+	      msg (M_INFO, "Current " PACKAGE_NAME " Statistics:");
 	      msg (M_INFO, " TUN/TAP read bytes:   " counter_format, tun_read_bytes);
 	      msg (M_INFO, " TUN/TAP write bytes:  " counter_format, tun_write_bytes);
 	      msg (M_INFO, " TCP/UDP read bytes:   " counter_format, link_read_bytes);
@@ -1885,7 +1927,7 @@ openvpn (const struct options *options,
 
 	      ASSERT (!to_tun.len);
 	      buf = read_link_buf;
-	      ASSERT (buf_init (&buf, EXTRA_FRAME (&frame)));
+	      ASSERT (buf_init (&buf, FRAME_HEADROOM (&frame)));
 
 	      status = link_socket_read (&link_socket, &buf, MAX_RW_SIZE_LINK (&frame), &from);
 
@@ -1990,6 +2032,7 @@ openvpn (const struct options *options,
 			  signal_received = SIGUSR1;
 			  msg (D_STREAM_ERRORS, "Fatal decryption error, restarting");
 			  signal_text = "decryption-error";
+			  mutex_unlock (L_TLS);
 			  break;
 			}
 		    }
@@ -2157,7 +2200,7 @@ openvpn (const struct options *options,
 #ifdef TUN_PASS_BUFFER
 	      read_tun_buffered (tuntap, &buf, MAX_RW_SIZE_TUN (&frame));
 #else
-	      ASSERT (buf_init (&buf, EXTRA_FRAME (&frame)));
+	      ASSERT (buf_init (&buf, FRAME_HEADROOM (&frame)));
 	      ASSERT (buf_safe (&buf, MAX_RW_SIZE_TUN (&frame)));
 	      buf.len = read_tun (tuntap, BPTR (&buf), MAX_RW_SIZE_TUN (&frame));
 #endif
@@ -2316,7 +2359,7 @@ openvpn (const struct options *options,
 	  /* TCP/UDP port ready to accept write */
 	  else if (SOCKET_ISSET (link_socket, writes))
 	    {
-	      if (to_link.len > 0 && to_link.len <= MAX_RW_SIZE_LINK (&frame))
+	      if (to_link.len > 0 && to_link.len <= EXPANDED_SIZE (&frame))
 		{
 		  /*
 		   * Setup for call to send/sendto which will send
@@ -2391,7 +2434,7 @@ openvpn (const struct options *options,
 		  msg (D_LINK_ERRORS, "TCP/UDP packet too large on write to %s (tried=%d,max=%d)",
 		       print_sockaddr (&to_link_addr),
 		       to_link.len,
-		       MAX_RW_SIZE_LINK (&frame));
+		       EXPANDED_SIZE (&frame));
 		}
 
 	      /*
@@ -2421,7 +2464,7 @@ openvpn (const struct options *options,
   if (options->inetd && (signal_received == SIGHUP || signal_received == SIGUSR1))
     {
       signal_received = SIGTERM;
-      msg (M_INFO, "OpenVPN started by inetd/xinetd cannot restart... Exiting.");
+      msg (M_INFO, PACKAGE_NAME " started by inetd/xinetd cannot restart... Exiting.");
     }
 
   if (free_to_link)
@@ -2504,7 +2547,7 @@ openvpn (const struct options *options,
 	  run_script (options->down_script,
 		      tuntap_actual,
 		      TUN_MTU_SIZE (&frame),
-		      MAX_RW_SIZE_LINK (&frame),
+		      EXPANDED_SIZE (&frame),
 		      print_in_addr_t (tuntap->local, true),
 		      print_in_addr_t (tuntap->remote_netmask, true),
 		      "init",
@@ -2518,7 +2561,7 @@ openvpn (const struct options *options,
 	    run_script (options->down_script,
 			tuntap->actual,
 			TUN_MTU_SIZE (&frame),
-			MAX_RW_SIZE_LINK (&frame),
+			EXPANDED_SIZE (&frame),
 			print_in_addr_t (tuntap->local, true),
 			print_in_addr_t (tuntap->remote_netmask, true),
 			"restart",
@@ -2619,6 +2662,14 @@ main (int argc, char *argv[])
     set_debug_level (options.verbosity);
     set_mute_cutoff (options.mute);
 
+    /*
+     * Possibly set --dev based on --dev-node.
+     * For example, if --dev-node /tmp/foo/tun, and --dev undefined,
+     * set --dev to tun.
+     */
+    if (!options.dev)
+      options.dev = dev_component_in_dev_node (options.dev_node);
+
     /*
      * OpenSSL info print mode?
      */
@@ -2643,14 +2694,6 @@ main (int argc, char *argv[])
 	goto exit;
       }
 
-    /*
-     * Possibly set --dev based on --dev-node.
-     * For example, if --dev-node /tmp/foo/tun, and --dev undefined,
-     * set --dev to tun.
-     */
-    if (!options.dev)
-      options.dev = dev_component_in_dev_node (options.dev_node);
-
     /*
      * Static pre-shared key generation mode?
      */
@@ -2727,6 +2770,33 @@ main (int argc, char *argv[])
       if (options.inetd && options.proto == PROTO_TCPv4_CLIENT)
 	msg (M_USAGE, "Options error: --proto tcp-client cannot be used with --inetd");
 
+      if (options.inetd == INETD_NOWAIT && options.proto != PROTO_TCPv4_SERVER)
+	msg (M_USAGE, "Options error: --inetd nowait can only be used with --proto tcp-server");
+
+      if (options.inetd == INETD_NOWAIT
+#if defined(USE_CRYPTO) && defined(USE_SSL)
+	  && !(options.tls_server || options.tls_client)
+#endif
+	  )
+	msg (M_USAGE, "Options error: --inetd nowait can only be used in TLS mode");
+
+      if (options.inetd == INETD_NOWAIT && dev != DEV_TYPE_TAP)
+	msg (M_USAGE, "Options error: --inetd nowait only makes sense in --dev tap mode");
+
+      /*
+       * In forking TCP server mode, you don't need to ifconfig
+       * the tap device (the assumption is that it will be bridged).
+       */
+      if (options.inetd == INETD_NOWAIT)
+	options.ifconfig_noexec = true;
+
+      /*
+       * Sanity check on TCP mode options
+       */
+
+      if (options.connect_retry_defined && options.proto != PROTO_TCPv4_CLIENT)
+	msg (M_USAGE, "Options error: --connect-retry doesn't make sense unless also used with --proto tcp-client");
+
       /*
        * Sanity check on MTU parameters
        */
@@ -2781,10 +2851,25 @@ main (int argc, char *argv[])
 #ifdef WIN32
       if (dev == DEV_TYPE_TUN && !(options.ifconfig_local && options.ifconfig_remote_netmask))
 	msg (M_USAGE, "Options error: On Windows, --ifconfig is required when --dev tun is used");
+
+      if ((options.tuntap_options.ip_win32_defined)
+	  && !(options.ifconfig_local && options.ifconfig_remote_netmask))
+	msg (M_USAGE, "Options error: On Windows, --ip-win32 doesn't make sense unless --ifconfig is also used");
+
+      if (options.tuntap_options.dhcp_options &&
+	  options.tuntap_options.ip_win32_type != IPW32_SET_DHCP_MASQ)
+	msg (M_USAGE, "Options error: --dhcp-options requires --ip-win32 dynamic");
+
+      if (options.tuntap_options.ip_win32_type == IPW32_SET_DHCP_MASQ
+	  && !options.route_delay_defined)
+	{
+	  options.route_delay_defined = true;
+	  options.route_delay = 10;
+	}
+
       if (options.ifconfig_noexec)
 	{
-	  options.tuntap_flags &= ~IP_SET_MASK;
-	  options.tuntap_flags |= IP_SET_MANUAL;
+	  options.tuntap_options.ip_win32_type = IPW32_SET_MANUAL;
 	  options.ifconfig_noexec = false;
 	}
 #endif
@@ -2803,6 +2888,12 @@ main (int argc, char *argv[])
       if (options.http_proxy_server && options.proto != PROTO_TCPv4_CLIENT)
 	msg (M_USAGE, "Options error: --http-proxy MUST be used in TCP Client mode (i.e. --proto tcp-client)");
 
+      if (options.http_proxy_server && options.socks_proxy_server)
+	msg (M_USAGE, "Options error: --http-proxy can not be used together with --socks-proxy");
+
+      if (options.socks_proxy_server && options.proto == PROTO_TCPv4_SERVER)
+	msg (M_USAGE, "Options error: --socks-proxy can not be used in TCP Server mode");
+
 #ifdef USE_CRYPTO
 
       if (first_time)
@@ -2899,6 +2990,7 @@ main (int argc, char *argv[])
 	struct packet_id_persist pid_persist;
 	struct route_list route_list;
 	struct http_proxy_info http_proxy;
+	struct socks_proxy_info socks_proxy;
 
 	/* print version number */
 	msg (M_INFO, "%s", title_string);
@@ -2909,6 +3001,7 @@ main (int argc, char *argv[])
 	packet_id_persist_init (&pid_persist);
 	clear_route_list (&route_list);
 	CLEAR (http_proxy);
+	CLEAR (socks_proxy);
 	if (options.http_proxy_server)
 	  {
 	    init_http_proxy (&http_proxy,
@@ -2919,8 +3012,16 @@ main (int argc, char *argv[])
 			     options.http_proxy_auth_file);
 	  }
 
+	if (options.socks_proxy_server)
+	  {
+	    init_socks_proxy (&socks_proxy,
+			      options.socks_proxy_server,
+			      options.socks_proxy_port,
+			      options.socks_proxy_retry);
+	  }
+
 	do {
-	  sig = openvpn (&options, &usa, &tuntap, &ks, &pid_persist, &route_list, &http_proxy, first_time);
+	  sig = openvpn (&options, &usa, &tuntap, &ks, &pid_persist, &route_list, &http_proxy, &socks_proxy, first_time);
 	  first_time = false;
 	} while (sig == SIGUSR1);
       }
@@ -2961,6 +3062,7 @@ test_crypto_thread (void *arg)
   struct packet_id_persist pid_persist;
   struct route_list route_list;
   struct http_proxy_info http_proxy;
+  struct socks_proxy_info socks_proxy;
   const struct options *opt = (struct options*) arg;
 
   /* print version number */
@@ -2973,7 +3075,8 @@ test_crypto_thread (void *arg)
   packet_id_persist_init (&pid_persist);
   clear_route_list (&route_list);
   CLEAR (http_proxy);
-  openvpn (opt, &usa, &tuntap, &ks, &pid_persist, &route_list, &http_proxy, false);
+  CLEAR (socks_proxy);
+  openvpn (opt, &usa, &tuntap, &ks, &pid_persist, &route_list, &http_proxy, &socks_proxy, false);
   return NULL;
 }
 #endif
