@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2003 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,11 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+ * 2004-01-28: Added Socks5 proxy support
+ *   (Christof Meerwald, http://cmeerw.org)
+ */
+
 #ifdef WIN32
 #include "config-win32.h"
 #else
@@ -79,12 +84,18 @@ static const char usage_message[] =
   "--remote host   : Remote host name or ip address.\n"
   "--proto p       : Use protocol p for communicating with peer.\n"
   "                  p = udp (default), tcp-server, or tcp-client\n"
+  "--connect-retry n : For --proto tcp-client, number of seconds to wait\n"
+  "                  between connection retries (default=%d).\n"
   "--http-proxy s p [up]: Connect to remote host through an HTTP proxy at address\n"
   "                  s and port p.  If proxy authentication is required, up is a\n"
   "                  file containing username/password on 2 lines.\n"
   "--http-proxy-retry : Retry indefinitely on HTTP proxy errors.\n"
+  "--socks-proxy s [p]: Connect to remote host through a Socks5 proxy at address\n"
+  "                  s and port p (default port = 1080).\n"
+  "--socks-proxy-retry : Retry indefinitely on Socks proxy errors.\n"
   "--resolv-retry n: If hostname resolve fails for --remote, retry\n"
   "                  resolve for n seconds before failing (disabled by default).\n"
+  "                  Set n=\"infinite\" to retry indefinitely.\n"
   "--float         : Allow remote to change its IP address/port, such as through\n"
   "                  DHCP (this is the default if --remote is not used).\n"
   "--ipchange cmd  : Execute shell command cmd on remote ip address initial\n"
@@ -123,8 +134,8 @@ static const char usage_message[] =
   "--route-up cmd  : Execute shell cmd after routes are added.\n"
   "--route-noexec  : Don't add routes automatically.  Instead pass routes to\n"
   "                  --route-up script using environmental variables.\n"
-  "--redirect-gateway : Automatically execute routing commands to cause all\n"
-  "                     outgoing IP traffic to be redirected into the VPN.\n"
+  "--redirect-gateway : (Experimental) Automatically execute routing commands to\n"
+  "                     redirect all outgoing IP traffic through the VPN.\n"
   "--setenv name value : Set a custom environmental variable to pass to script.\n"
   "--shaper n      : Restrict output to peer to n bytes per second.\n"
   "--inactive n    : Exit after n seconds of inactivity on TUN/TAP device.\n"
@@ -179,8 +190,8 @@ static const char usage_message[] =
   "--daemon [name] : Become a daemon after initialization.\n"
   "                  The optional 'name' parameter will be passed\n"
   "                  as the program name to the system logger.\n"
-  "--inetd [name]  : Run as an inetd or xinetd server.  See --daemon\n"
-  "                  above for a description of the 'name' parameter.\n"
+  "--inetd [name] ['wait'|'nowait'] : Run as an inetd or xinetd server.\n"
+  "                  See --daemon above for a description of the 'name' parm.\n"
   "--log file      : Output log to file which is created/truncated on open.\n"
   "--log-append file : Append log to file, or create file if nonexistent.\n"
   "--writepid file : Write main process ID to file.\n"
@@ -300,7 +311,28 @@ static const char usage_message[] =
   "--show-adapters : Show all TAP-Win32 adapters.\n"
   "--ip-win32 method : When using --ifconfig on Windows, set TAP-Win32 adapter\n"
   "                    IP address using method = manual, netsh, ipapi, or\n"
-  "                    dynamic (default = ipapi).\n"
+  "                    dynamic (default = dynamic).\n"
+  "                    Dynamic method allows two optional parameters:\n"
+  "                    offset: DHCP server address offset (> -256 and < 256).\n"
+  "                            If 0, use network address, if >0, take nth\n"
+  "                            address forward from network address, if <0,\n"
+  "                            take nth address backward from broadcast\n"
+  "                            address.\n"
+  "                            Default is 0.\n"
+  "                    lease-time: Lease time in seconds.\n"
+  "                                Default is one year.\n"
+  "--dhcp-option type [parm] : Set extended TAP-Win32 properties, must\n"
+  "                    be used with --ip-win32 dynamic.  For options\n"
+  "                    which allow multiple addresses,\n"
+  "                    --dhcp-option must be repeated.\n"
+  "                    DOMAIN name : Set DNS suffix\n"
+  "                    DNS addr    : Set domain name server address(es)\n"
+  "                    NTP         : Set NTP server address(es)\n"
+  "                    NBDD        : Set NBDD server address(es)\n"
+  "                    WINS addr   : Set WINS server address(es)\n"
+  "                    NBT type    : Set NetBIOS over TCP/IP Node type\n"
+  "                                  1: B, 2: P, 4: M, 8: H\n"
+  "                    NBS id      : Set NetBIOS scope ID\n"
   "--tap-sleep n   : Sleep for n seconds after TAP adapter open before\n"
   "                  attempting to set adapter properties.\n"
   "--show-valid-subnets : Show valid subnets for --dev tun emulation.\n" 
@@ -332,6 +364,7 @@ init_options (struct options *o)
 {
   CLEAR (*o);
   o->proto = PROTO_UDPv4;
+  o->connect_retry_seconds = 5;
 #ifdef TUNSETPERSIST
   o->persist_mode = 1;
 #endif
@@ -346,7 +379,9 @@ init_options (struct options *o)
   o->comp_lzo_adaptive = true;
 #endif
 #ifdef WIN32
-  o->tuntap_flags = (IP_SET_IPAPI & IP_SET_MASK);
+  o->tuntap_options.ip_win32_type = IPW32_SET_DHCP_MASQ;
+  o->tuntap_options.dhcp_lease_time = 31536000; /* one year */
+  o->tuntap_options.dhcp_masq_offset = 0;       /* use network address as internal DHCP server address */
 #endif
 #ifdef USE_CRYPTO
   o->ciphername = "BF-CBC";
@@ -359,7 +394,11 @@ init_options (struct options *o)
   o->use_iv = true;
   o->key_direction = KEY_DIRECTION_BIDIRECTIONAL;
 #ifdef USE_SSL
+#ifdef KEY_METHOD_DEFAULT_2
+  o->key_method = 2;
+#else
   o->key_method = 1;
+#endif
   o->tls_timeout = 2;
   o->renegotiate_seconds = 3600;
   o->handshake_window = 60;
@@ -386,6 +425,60 @@ setenv_settings (const struct options *o)
   setenv_int ("remote_port", o->remote_port);
 }
 
+#ifdef WIN32
+
+static void
+show_dhcp_option_addrs (const char *name, const in_addr_t *array, int len)
+{
+  int i;
+  for (i = 0; i < len; ++i)
+    {
+      msg (D_SHOW_PARMS, "  %s[%d] = %s",
+	   name,
+	   i,
+	   print_in_addr_t (array[i], false));
+    }
+}
+
+static void
+show_tuntap_options (const struct tuntap_options *o)
+{
+  SHOW_BOOL (ip_win32_defined);
+  SHOW_INT (ip_win32_type);
+  SHOW_INT (dhcp_masq_offset);
+  SHOW_INT (dhcp_lease_time);
+  SHOW_INT (tap_sleep);
+  SHOW_BOOL (dhcp_options);
+  SHOW_STR (domain);
+  SHOW_STR (netbios_scope);
+  SHOW_INT (netbios_node_type);
+
+  show_dhcp_option_addrs ("DNS", o->dns, o->dns_len);
+  show_dhcp_option_addrs ("WINS", o->wins, o->wins_len);
+  show_dhcp_option_addrs ("NTP", o->ntp, o->ntp_len);
+  show_dhcp_option_addrs ("NBDD", o->nbdd, o->nbdd_len);
+}
+
+static void
+dhcp_option_address_parse (const char *name, const char *parm, in_addr_t *array, int *len)
+{
+  if (*len >= N_DHCP_ADDR)
+    msg (M_USAGE, "--dhcp-option %s: maximum of %d %s servers can be specified",
+	 name,
+	 N_DHCP_ADDR,
+	 name);
+
+  array[(*len)++] = getaddr (GETADDR_FATAL
+			   | GETADDR_HOST_ORDER
+			   | GETADDR_FATAL_ON_SIGNAL,
+			   parm,
+			   0,
+			   NULL,
+			   NULL);
+}
+
+#endif
+
 void
 show_settings (const struct options *o)
 {
@@ -461,6 +554,7 @@ show_settings (const struct options *o)
 #endif
 
   SHOW_INT (resolve_retry_seconds);
+  SHOW_INT (connect_retry_seconds);
 
   SHOW_STR (username);
   SHOW_STR (groupname);
@@ -471,13 +565,12 @@ show_settings (const struct options *o)
   SHOW_STR (down_script);
   SHOW_BOOL (up_restart);
   SHOW_BOOL (daemon);
-  SHOW_BOOL (inetd);
+  SHOW_INT (inetd);
   SHOW_BOOL (log);
   SHOW_INT (nice);
   SHOW_INT (verbosity);
   SHOW_INT (mute);
   SHOW_BOOL (gremlin);
-  SHOW_UINT (tuntap_flags);
 
   SHOW_BOOL (occ);
 
@@ -487,6 +580,10 @@ show_settings (const struct options *o)
   SHOW_STR (http_proxy_auth_file);
   SHOW_BOOL (http_proxy_retry);
 
+  SHOW_STR (socks_proxy_server);
+  SHOW_INT (socks_proxy_port);
+  SHOW_BOOL (socks_proxy_retry);
+
 #ifdef USE_LZO
   SHOW_BOOL (comp_lzo);
   SHOW_BOOL (comp_lzo_adaptive);
@@ -541,6 +638,10 @@ show_settings (const struct options *o)
   SHOW_STR (tls_auth_file);
 #endif
 #endif
+
+#ifdef WIN32
+  show_tuntap_options (&o->tuntap_options);
+#endif
 }
 
 #undef SHOW_PARM
@@ -607,8 +708,8 @@ options_string (const struct options *o,
    */
 
   buf_printf (&out, ",dev-type %s", dev_type_string (o->dev, o->dev_type));
-  buf_printf (&out, ",link-mtu %d", MAX_RW_SIZE_LINK(frame));
-  buf_printf (&out, ",tun-mtu %d", MAX_RW_SIZE_TUN(frame));
+  buf_printf (&out, ",link-mtu %d", EXPANDED_SIZE (frame));
+  buf_printf (&out, ",tun-mtu %d", PAYLOAD_SIZE (frame));
   buf_printf (&out, ",proto %s", proto2ascii (proto_remote (o->proto, remote), true));
   if (o->tun_ipv6)
     buf_printf (&out, ",tun-ipv6");
@@ -724,7 +825,7 @@ options_cmp_equal (char *actual, const char *expected, size_t actual_n)
 #ifndef STRICT_OPTIONS_CHECK
       if (strncmp (actual, expected, 2))
 	{
-	  msg (D_SHOW_OCC, "NOTE: failed to perform options consistency check between peers because of OpenVPN version differences -- you can disable the options consistency check with --disable-occ (Required for TLS connections between OpenVPN 1.3.x and later versions).  Actual Remote Options: '%s'.  Expected Remote Options: '%s'", actual, expected);
+	  msg (D_SHOW_OCC, "NOTE: failed to perform options consistency check between peers because of " PACKAGE_NAME " version differences -- you can disable the options consistency check with --disable-occ (Required for TLS connections between " PACKAGE_NAME " 1.3.x and later versions).  Actual Remote Options: '%s'.  Expected Remote Options: '%s'", actual, expected);
 	  return true;
 	}
       else
@@ -784,7 +885,9 @@ usage (void)
 
 #if defined(USE_CRYPTO) && defined(USE_SSL)
   fprintf (fp, usage_message,
-	   title_string, o.local_port, o.remote_port,
+	   title_string,
+	   o.connect_retry_seconds,
+	   o.local_port, o.remote_port,
 	   TAP_MTU_DEFAULT, TAP_MTU_EXTRA_DEFAULT, LINK_MTU_DEFAULT,
 	   o.verbosity,
 	   o.authname, o.ciphername,
@@ -793,14 +896,18 @@ usage (void)
 	   o.handshake_window, o.transition_window);
 #elif defined(USE_CRYPTO)
   fprintf (fp, usage_message,
-	   title_string, o.local_port, o.remote_port,
+	   title_string,
+	   o.connect_retry_seconds,
+	   o.local_port, o.remote_port,
 	   TAP_MTU_DEFAULT, TAP_MTU_EXTRA_DEFAULT, LINK_MTU_DEFAULT,
 	   o.verbosity,
 	   o.authname, o.ciphername,
            o.replay_window, o.replay_time);
 #else
   fprintf (fp, usage_message,
-	   title_string, o.local_port, o.remote_port,
+	   title_string,
+	   o.connect_retry_seconds,
+	   o.local_port, o.remote_port,
 	   TAP_MTU_DEFAULT, TAP_MTU_EXTRA_DEFAULT, LINK_MTU_DEFAULT,
 	   o.verbosity);
 #endif
@@ -820,7 +927,7 @@ static void
 usage_version (void)
 {
   msg (M_INFO|M_NOPREFIX, "%s", title_string);
-  msg (M_INFO|M_NOPREFIX, "Copyright (C) 2002-2003 James Yonan <jim@yonan.net>");
+  msg (M_INFO|M_NOPREFIX, "Copyright (C) 2002-2004 James Yonan <jim@yonan.net>");
   openvpn_exit (OPENVPN_EXIT_STATUS_USAGE); /* exit point */
 }
 
@@ -861,75 +968,101 @@ space (char c)
 static int
 parse_line (char *line, char *p[], int n, const char *file, int line_num)
 {
+  const int STATE_INITIAL = 0;
+  const int STATE_READING_QUOTED_PARM = 1;
+  const int STATE_READING_UNQUOTED_PARM = 2;
+  const int STATE_DONE = 3;
+
   int ret = 0;
   char *c = line;
-  char *start = NULL;
+  int state = STATE_INITIAL;
+  bool backslash = false;
+  char in, out;
 
-  /*
-   * Parse states:
-   * 0 -- Initial
-   * 1 -- Reading non-quoted parm
-   * 2 -- Leading quote
-   * 3 -- Reading quoted parm
-   * 4 -- First char after parm
-   */
-  int state = 0;
+  char parm[256];
+  unsigned int parm_len = 0;
 
   do
     {
-      if (state == 0)
+      in = *c;
+      out = 0;
+
+      if (!backslash && in == '\\')
 	{
-	  if (!space (*c))
+	  backslash = true;
+	}
+      else
+	{
+	  if (state == STATE_INITIAL)
 	    {
-	      if (*c == ';' || *c == '#') /* comment */
-		break;
-	      if (*c == '\"')
-		state = 2;
-	      else
+	      if (!space (in))
 		{
-		  start = c;
-		  state = 1;
+		  if (in == ';' || in == '#') /* comment */
+		    break;
+		  if (!backslash && in == '\"')
+		    state = STATE_READING_QUOTED_PARM;
+		  else
+		    {
+		      out = in;
+		      state = STATE_READING_UNQUOTED_PARM;
+		    }
 		}
 	    }
+	  else if (state == STATE_READING_UNQUOTED_PARM)
+	    {
+	      if (!backslash && space (in))
+		state = STATE_DONE;
+	      else
+		out = in;
+	    }
+	  else if (state == STATE_READING_QUOTED_PARM)
+	    {
+	      if (!backslash && in == '\"')
+		state = STATE_DONE;
+	      else
+		out = in;
+	    }
+	  if (state == STATE_DONE)
+	    {
+	      ASSERT (parm_len > 0);
+	      p[ret] = gc_malloc (parm_len + 1);
+	      memcpy (p[ret], parm, parm_len);
+	      p[ret][parm_len] = '\0';
+	      state = 0;
+	      parm_len = 0;
+	      ++ret;
+	    }
+	  backslash = false;
 	}
-      else if (state == 1)
+
+      /* store parameter character */
+      if (out)
 	{
-	  if (space (*c))
-	    state = 4;
-	}
-      else if (state == 2)
-	{
-	  start = c;
-	  state = 3;
-	}
-      else if (state == 3)
-	{
-	  if (*c == '\"')
-	    state = 4;
-	}
-      if (state == 4)
-	{
-	  const int len = (int) (c - start);
-	  ASSERT (len > 0);
-	  p[ret] = gc_malloc (len + 1);
-	  memcpy (p[ret], start, len);
-	  p[ret][len] = '\0';
-	  state = 0;
-	  if (++ret >= n)
-	    break;
+	  if (parm_len >= SIZE (parm))
+	    {
+	      parm[SIZE (parm) - 1] = 0;
+	      msg (M_USAGE, "Parameter at %s:%d is too long (%d chars max): %s",
+		   file, line_num, (int) SIZE (parm), parm);
+	    }
+	  parm[parm_len++] = out;
 	}
+
+      /* avoid overflow if too many parms in one config file line */
+      if (ret >= n)
+	break;
+
     } while (*c++ != '\0');
 
-  if (state == 2 || state == 3)
+  if (state == STATE_READING_QUOTED_PARM)
 	msg (M_FATAL, "No closing quotation (\") in %s:%d", file, line_num);
-  if (state)
+  if (state != STATE_INITIAL)
 	msg (M_FATAL, "Residual parse state (%d) in %s:%d", state, file, line_num);
 #if 0
   {
     int i;
     for (i = 0; i < ret; ++i)
       {
-	msg (M_INFO, "%s:%d ARG[%d] '%s'", file, line_num, i, p[i]);
+	msg (M_INFO|M_NOPREFIX, "%s:%d ARG[%d] '%s'", file, line_num, i, p[i]);
       }
   }
 #endif
@@ -1082,7 +1215,16 @@ add_option (struct options *options, int i, char *p[],
   else if (streq (p[0], "resolv-retry") && p[1])
     {
       ++i;
-      options->resolve_retry_seconds = positive (atoi (p[1]));
+      if (streq (p[1], "infinite"))
+	options->resolve_retry_seconds = 1000000000;
+      else
+	options->resolve_retry_seconds = positive (atoi (p[1]));
+    }
+  else if (streq (p[0], "connect-retry") && p[1])
+    {
+      ++i;
+      options->connect_retry_seconds = positive (atoi (p[1]));
+      options->connect_retry_defined = true;
     }
   else if (streq (p[0], "ipchange") && p[1])
     {
@@ -1155,11 +1297,46 @@ add_option (struct options *options, int i, char *p[],
     {
       if (!options->inetd)
 	{
-	  options->inetd = true;
+	  int z;
+	  const char *name = NULL;
+	  const char *opterr = "Options Error: when --inetd is used with two parameters, one of them must be 'wait' or 'nowait' and the other must be a daemon name to use for system logging";
+
+	  options->inetd = -1;
+
+	  for (z = 1; z <= 2; ++z)
+	    {
+	      if (p[z])
+		{
+		  ++i;
+		  if (streq (p[z], "wait"))
+		    {
+		      if (options->inetd != -1)
+			msg (M_USAGE, opterr);
+		      else
+			options->inetd = INETD_WAIT;
+		    }
+		  else if (streq (p[z], "nowait"))
+		    {
+		      if (options->inetd != -1)
+			msg (M_USAGE, opterr);
+		      else
+			options->inetd = INETD_NOWAIT;
+		    }
+		  else
+		    {
+		      if (name != NULL)
+			msg (M_USAGE, opterr);
+		      name = p[z];
+		    }
+		}
+	    }
+
+	  /* default */
+	  if (options->inetd == -1)
+	    options->inetd = INETD_WAIT;
+
 	  save_inetd_socket_descriptor ();
-	  open_syslog (p[1]);
-	  if (p[1])
-	    ++i;
+	  open_syslog (name);
 	}
     }
   else if (streq (p[0], "log") && p[1])
@@ -1314,6 +1491,27 @@ add_option (struct options *options, int i, char *p[],
     {
       options->http_proxy_retry = true;
     }
+  else if (streq (p[0], "socks-proxy") && p[1])
+    {
+      ++i;
+      options->socks_proxy_server = p[1];
+
+      if (p[2])
+	{
+	  ++i;
+          options->socks_proxy_port = atoi (p[2]);
+          if (options->socks_proxy_port <= 0)
+	    msg (M_USAGE, "Bad socks-proxy port number: %s", p[2]);
+	}
+      else
+	{
+	  options->socks_proxy_port = 1080;
+	}
+    }
+  else if (streq (p[0], "socks-proxy-retry"))
+    {
+      options->socks_proxy_retry = true;
+    }
   else if (streq (p[0], "ping") && p[1])
     {
       ++i;
@@ -1419,20 +1617,96 @@ add_option (struct options *options, int i, char *p[],
   else if (streq (p[0], "ip-win32") && p[1])
     {
       const int index = ascii2ipset (p[1]);
+      struct tuntap_options *to = &options->tuntap_options;
       ++i;
+
+      to->ip_win32_defined = true;
+ 
       if (index < 0)
 	msg (M_USAGE,
 	     "Bad --ip-win32 method: '%s'.  Allowed methods: %s",
 	     p[1],
 	     ipset2ascii_all());
 
-#if 1
-      if (index == IP_SET_DHCP)
-	msg (M_FATAL|M_NOPREFIX, "Sorry but '--ip-win32 dynamic' has not been implemented yet -- try one of the other three methods.");
-#endif
+      to->ip_win32_type = index;
 
-      options->tuntap_flags &= ~IP_SET_MASK;
-      options->tuntap_flags |= (index & IP_SET_MASK);
+      if (to->ip_win32_type == IPW32_SET_DHCP_MASQ)
+	{
+	  if (p[2])
+	    {
+	      const int min_lease = 30;
+	      int offset = atoi (p[2]);
+
+	      ++i;
+	      to->dhcp_masq_custom_offset = true;
+
+	      if (!(offset > -256 && offset < 256))
+		msg (M_USAGE, "--ip-win32 dynamic [offset] [lease-time]: offset (%d) must be > -256 and < 256", offset);
+
+	      to->dhcp_masq_offset = offset;
+
+	      if (p[3])
+		{
+		  const int min_lease = 30;
+		  int lease_time;
+		  ++i;
+		  lease_time = atoi (p[3]);
+		  if (lease_time < min_lease)
+		    msg (M_USAGE, "--ip-win32 dynamic [offset] [lease-time]: lease time parameter (%d) must be at least %d seconds", lease_time, min_lease);
+		  to->dhcp_lease_time = lease_time;
+		}
+	    }
+	}
+    }
+  else if (streq (p[0], "dhcp-option") && p[1])
+    {
+      struct tuntap_options *o = &options->tuntap_options;
+      ++i;
+      o->dhcp_options = true;
+
+      if (streq (p[1], "DOMAIN") && p[2])
+	{
+	  ++i;
+	  o->domain = p[2];
+	}
+      else if (streq (p[1], "NBS") && p[2])
+	{
+	  ++i;
+	  o->netbios_scope = p[2];
+	}
+      else if (streq (p[1], "NBT") && p[2])
+	{
+	  int t;
+	  ++i;
+	  t = atoi (p[2]);
+	  if (!(t == 1 || t == 2 || t == 4 || t == 8))
+	    msg (M_USAGE, "--dhcp-option NBT: parameter (%d) must be 1, 2, 4, or 8", t);
+	  o->netbios_node_type = t;
+	}
+      else if (streq (p[1], "DNS") && p[2])
+	{
+	  ++i;
+	  dhcp_option_address_parse ("DNS", p[2], o->dns, &o->dns_len);
+	}
+      else if (streq (p[1], "WINS") && p[2])
+	{
+	  ++i;
+	  dhcp_option_address_parse ("WINS", p[2], o->wins, &o->wins_len);
+	}
+      else if (streq (p[1], "NTP") && p[2])
+	{
+	  ++i;
+	  dhcp_option_address_parse ("NTP", p[2], o->ntp, &o->ntp_len);
+	}
+      else if (streq (p[1], "NBDD") && p[2])
+	{
+	  ++i;
+	  dhcp_option_address_parse ("NBDD", p[2], o->nbdd, &o->nbdd_len);
+	}
+      else
+	{
+	  msg (M_USAGE, "--dhcp-option: unknown option type '%s' or missing parameter", p[1]);
+	}
     }
   else if (streq (p[0], "show-adapters"))
     {
@@ -1446,8 +1720,7 @@ add_option (struct options *options, int i, char *p[],
       s = atoi (p[1]);
       if (s < 0 || s >= 256)
 	msg (M_FATAL, "--tap-sleep parameter must be between 0 and 255");
-      options->tuntap_flags &= ~(TUNTAP_SLEEP_MASK << TUNTAP_SLEEP_SHIFT);
-      options->tuntap_flags |= (s << TUNTAP_SLEEP_SHIFT);
+      options->tuntap_options.tap_sleep = s;
     }
   else if (streq (p[0], "show-valid-subnets"))
     {
