@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2003 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,6 +45,18 @@
 
 #include "memdbg.h"
 
+#if defined(TARGET_FREEBSD)
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+
+struct {
+  struct rt_msghdr m_rtm;
+  char       m_space[512];
+} m_rtmsg;
+
+#endif
+
 static void add_route (struct route *r);
 static void delete_route (const struct route *r);
 static bool get_default_gateway (in_addr_t *ret);
@@ -97,7 +109,7 @@ get_special_addr (const struct route_special_addr *spec,
 	*out = spec->remote_endpoint;
       else
 	{
-	  msg (M_INFO, "OpenVPN ROUTE: vpn_gateway undefined");
+	  msg (M_INFO, PACKAGE_NAME " ROUTE: vpn_gateway undefined");
 	  *status = false;
 	}
       return true;
@@ -108,7 +120,7 @@ get_special_addr (const struct route_special_addr *spec,
 	*out = spec->net_gateway;
       else
 	{
-	  msg (M_INFO, "OpenVPN ROUTE: net_gateway undefined -- unable to get default gateway from system");
+	  msg (M_INFO, PACKAGE_NAME " ROUTE: net_gateway undefined -- unable to get default gateway from system");
 	  *status = false;
 	}
       return true;
@@ -119,7 +131,7 @@ get_special_addr (const struct route_special_addr *spec,
 	*out = spec->remote_host;
       else
 	{
-	  msg (M_INFO, "OpenVPN ROUTE: remote_host undefined");
+	  msg (M_INFO, PACKAGE_NAME " ROUTE: remote_host undefined");
 	  *status = false;
 	}
       return true;
@@ -201,7 +213,7 @@ init_route (struct route *r,
 	r->gateway = spec->remote_endpoint;
       else
 	{
-	  msg (M_WARN, "OpenVPN ROUTE: OpenVPN needs a gateway parameter for a --route option and no default was specified by either --route-gateway or --ifconfig options");
+	  msg (M_WARN, PACKAGE_NAME " ROUTE: " PACKAGE_NAME " needs a gateway parameter for a --route option and no default was specified by either --route-gateway or --ifconfig options");
 	  goto fail;
 	}
     }
@@ -215,7 +227,7 @@ init_route (struct route *r,
       r->metric = atoi (ro->metric);
       if (r->metric < 0)
 	{
-	  msg (M_WARN, "OpenVPN ROUTE: route metric for network %s (%s) must be >= 0",
+	  msg (M_WARN, PACKAGE_NAME " ROUTE: route metric for network %s (%s) must be >= 0",
 	       ro->network,
 	       ro->metric);
 	  goto fail;
@@ -233,7 +245,7 @@ init_route (struct route *r,
   return true;
 
  fail:
-  msg (M_WARN, "OpenVPN ROUTE: failed to parse/resolve route for host/network: %s",
+  msg (M_WARN, PACKAGE_NAME " ROUTE: failed to parse/resolve route for host/network: %s",
        ro->network);
   r->defined = false;
   return false;
@@ -248,7 +260,7 @@ add_route_to_option_list (struct route_option_list *l,
 {
   struct route_option *ro;
   if (l->n >= MAX_ROUTES)
-    msg (M_FATAL, "OpenVPN ROUTE: cannot add more than %d routes",
+    msg (M_FATAL, PACKAGE_NAME " ROUTE: cannot add more than %d routes",
 	 MAX_ROUTES);
   ro = &l->routes[l->n];
   ro->network = network;
@@ -282,7 +294,7 @@ init_route_list (struct route_list *rl,
     }
 
   rl->spec.net_gateway_defined = get_default_gateway (&rl->spec.net_gateway);
-  if (rl->spec.remote_endpoint_defined)
+  if (rl->spec.net_gateway_defined)
     {
       setenv_route_addr ("net_gateway", rl->spec.net_gateway, -1);
     }
@@ -305,7 +317,7 @@ init_route_list (struct route_list *rl,
 	}
       else
 	{
-	  msg (M_WARN, "OpenVPN ROUTE: failed to parse/resolve default gateway: %s",
+	  msg (M_WARN, PACKAGE_NAME " ROUTE: failed to parse/resolve default gateway: %s",
 	       remote_endpoint);
 	  ret = false;
 	}
@@ -545,14 +557,22 @@ add_route (struct route *r)
   gateway = print_in_addr_t (r->gateway, false);
 
 #if defined(TARGET_LINUX)
+#ifdef CONFIG_FEATURE_IPROUTE
+  buf_printf (&buf, IPROUTE_PATH " route add %s/%d via %s",
+	      network,
+	      count_netmask_bits(netmask),
+	      gateway);
+  if (r->metric_defined)
+    buf_printf (&buf, " metric %d", r->metric);
 
+#else
   buf_printf (&buf, ROUTE_PATH " add -net %s netmask %s gw %s",
 	      network,
 	      netmask,
 	      gateway);
   if (r->metric_defined)
     buf_printf (&buf, " metric %d", r->metric);
-
+#endif  /*CONFIG_FEATURE_IPROUTE*/
   msg (D_ROUTE, "%s", BSTR (&buf));
   status = system_check (BSTR (&buf), "ERROR: Linux route add command failed", false);
 
@@ -650,11 +670,16 @@ delete_route (const struct route *r)
   gateway = print_in_addr_t (r->gateway, false);
 
 #if defined(TARGET_LINUX)
+#ifdef CONFIG_FEATURE_IPROUTE
+  buf_printf (&buf, IPROUTE_PATH " route del %s/%d",
+	      network,
+	      count_netmask_bits(netmask));
+#else
 
   buf_printf (&buf, ROUTE_PATH " del -net %s netmask %s",
 	      network,
 	      netmask);
-
+#endif /*CONFIG_FEATURE_IPROUTE*/
   msg (D_ROUTE, "%s", BSTR (&buf));
   system_check (BSTR (&buf), "ERROR: Linux route delete command failed", false);
 
@@ -798,6 +823,96 @@ get_default_gateway (in_addr_t *ret)
   return false;
 }
 
+#elif defined(TARGET_FREEBSD)
+
+#define ROUNDUP(a) \
+        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
+
+static bool
+get_default_gateway (in_addr_t *ret)
+{
+  int s, seq, l, pid, rtm_addrs, i;
+  struct sockaddr so_dst, so_mask;
+  char *cp = m_rtmsg.m_space; 
+  struct sockaddr *gate = NULL, *sa;
+  struct  rt_msghdr *rtm_aux;
+
+#define NEXTADDR(w, u) \
+        if (rtm_addrs & (w)) {\
+            l = ROUNDUP(u.sa_len); memmove(cp, &(u), l); cp += l;\
+        }
+
+#define ADVANCE(x, n) (x += ROUNDUP((n)->sa_len))
+
+#define rtm m_rtmsg.m_rtm
+
+  pid = getpid();
+  seq = 0;
+  rtm_addrs = RTA_DST | RTA_NETMASK;
+
+  bzero(&so_dst, sizeof(so_dst));
+  bzero(&so_mask, sizeof(so_mask));
+  bzero(&rtm, sizeof(struct rt_msghdr));
+
+  rtm.rtm_type = RTM_GET;
+  rtm.rtm_flags = RTF_UP | RTF_GATEWAY;
+  rtm.rtm_version = RTM_VERSION;
+  rtm.rtm_seq = ++seq;
+  rtm.rtm_addrs = rtm_addrs; 
+
+  so_dst.sa_family = AF_INET;
+  so_dst.sa_len = sizeof(struct sockaddr_in);
+  so_mask.sa_family = AF_INET;
+  so_mask.sa_len = sizeof(struct sockaddr_in);
+
+  NEXTADDR(RTA_DST, so_dst);
+  NEXTADDR(RTA_NETMASK, so_mask);
+
+  rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
+
+  s = socket(PF_ROUTE, SOCK_RAW, 0);
+
+  if (write(s, (char *)&m_rtmsg, l) < 0) {
+                warn("writing to routing socket");
+                return false;
+  }
+
+  do {
+        l = read(s, (char *)&m_rtmsg, sizeof(m_rtmsg));
+  } while (l > 0 && (rtm.rtm_seq != seq || rtm.rtm_pid != pid));
+                        
+
+  rtm_aux = &rtm;
+
+  cp = ((char *)(rtm_aux + 1));
+  if (rtm_aux->rtm_addrs) {
+        for (i = 1; i; i <<= 1)
+             if (i & rtm_aux->rtm_addrs) {
+                   sa = (struct sockaddr *)cp;
+		   if( i == RTA_GATEWAY )
+                      gate = sa;
+                   ADVANCE(cp, sa);
+	     }
+  }
+  else
+	return false;
+
+
+  if( gate != NULL )
+  {
+	*ret = ntohl(((struct sockaddr_in *)gate)->sin_addr.s_addr);
+#if 1
+        msg (M_INFO, "gw %s",
+                 print_in_addr_t ((in_addr_t) *ret, false));
+#endif
+
+	return true;
+  }
+  else
+	return false;
+}
+
+
 #else
 
 static bool
