@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2003 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -167,8 +167,9 @@ tls_init_control_channel_frame_parameters(const struct frame *data_channel_frame
    * if --tls-auth is enabled.
    */
 
-  /* inherit link MTU from data channel */
+  /* inherit link MTU and extra_link from data channel */
   frame->link_mtu = data_channel_frame->link_mtu;
+  frame->extra_link = data_channel_frame->extra_link;
 
   /* set extra_frame */
   tls_adjust_frame_parameters (frame);
@@ -322,6 +323,13 @@ verify_callback (int preverify_ok, X509_STORE_CTX * ctx)
   verify_maxlevel = max_int (verify_maxlevel, ctx->error_depth);
   openvpn_snprintf (envname, sizeof(envname), "tls_id_%d", ctx->error_depth);
   setenv_str (envname, txt);
+
+  /* export serial number as environmental variable */
+  {
+    const int serial = (int) ASN1_INTEGER_get (X509_get_serialNumber (ctx->current_cert));
+    openvpn_snprintf (envname, sizeof(envname), "tls_serial_%d", ctx->error_depth);
+    setenv_int (envname, serial);
+  }
   
   if (verify_x509name)
     if (ctx->error_depth == 0)
@@ -345,7 +353,10 @@ verify_callback (int preverify_ok, X509_STORE_CTX * ctx)
       setenv_str ("script_type", "tls-verify");
 
       buf_set_write (&out, (uint8_t*)command, sizeof (command));
-      buf_printf (&out, "%s %d %s", verify_command, ctx->error_depth, txt);
+      buf_printf (&out, "%s %d %s",
+		  verify_command,
+		  ctx->error_depth,
+		  txt);
       msg (D_TLS_DEBUG, "executing verify command: %s", command);
       ret = openvpn_system (command);
       if (system_ok (ret))
@@ -484,17 +495,9 @@ init_ssl (bool server,
   /* Set callback for getting password from user to decrypt private key */
   SSL_CTX_set_default_passwd_cb (ctx, pem_password_callback);
 
-#if 1
   /* Load Certificate */
   if (!SSL_CTX_use_certificate_file (ctx, cert_file, SSL_FILETYPE_PEM))
     msg (M_SSLERR, "Cannot load certificate file %s", cert_file);
-#else
-  /* Load Certificate -- for some reason, this function sometimes
-     inexplicably fails during restarts with a PEM_R_NO_START_LINE
-     error. */
-  if (!SSL_CTX_use_certificate_chain_file (ctx, cert_file))
-    msg (M_SSLERR, "Cannot load certificate chain file %s", cert_file);
-#endif
 
   /* Load Private Key */
   if (!SSL_CTX_use_PrivateKey_file (ctx, priv_key_file, SSL_FILETYPE_PEM))
@@ -509,7 +512,6 @@ init_ssl (bool server,
   if (!SSL_CTX_load_verify_locations (ctx, ca_file, NULL))
     msg (M_SSLERR, "Cannot load CA certificate file %s (SSL_CTX_load_verify_locations)", ca_file);
 
-#if 1
   /* Load names of CAs from file and use it as a client CA list */
   {
     STACK_OF(X509_NAME) *cert_names;
@@ -518,7 +520,10 @@ init_ssl (bool server,
       msg (M_SSLERR, "Cannot load CA certificate file %s (SSL_load_client_CA_file)", ca_file);
     SSL_CTX_set_client_CA_list (ctx, cert_names);
   }
-#endif
+
+  /* Enable the use of certificate chains */
+  if (!SSL_CTX_use_certificate_chain_file (ctx, cert_file))
+    msg (M_SSLERR, "Cannot load certificate chain file %s (SSL_use_certificate_chain_file)", cert_file);
 
   /* Require peer certificate verification */
   SSL_CTX_set_verify (ctx, SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
@@ -967,9 +972,9 @@ key_state_init (struct tls_session *session, struct key_state *ks,
   ks->plaintext_write_buf = alloc_buf (PLAINTEXT_BUFFER_SIZE);
   ks->ack_write_buf = alloc_buf (BUF_SIZE (&session->opt->frame));
   reliable_init (&ks->send_reliable, BUF_SIZE (&session->opt->frame),
-		 EXTRA_FRAME (&session->opt->frame), TLS_RELIABLE_N_SEND_BUFFERS);
+		 FRAME_HEADROOM (&session->opt->frame), TLS_RELIABLE_N_SEND_BUFFERS);
   reliable_init (&ks->rec_reliable, BUF_SIZE (&session->opt->frame),
-		 EXTRA_FRAME (&session->opt->frame), TLS_RELIABLE_N_REC_BUFFERS);
+		 FRAME_HEADROOM (&session->opt->frame), TLS_RELIABLE_N_REC_BUFFERS);
   reliable_set_timeout (&ks->send_reliable, session->opt->packet_timeout);
 
   /* init packet ID tracker */
@@ -1573,7 +1578,7 @@ generate_key_expansion (struct key_ctx_bi *key,
   /* compute master secret */
   openvpn_PRF (key_src->client.pre_master,
 	       sizeof(key_src->client.pre_master),
-	       "OpenVPN master secret",
+	       PACKAGE_NAME " master secret",
 	       key_src->client.random1,
 	       sizeof(key_src->client.random1),
 	       key_src->server.random1,
@@ -1586,7 +1591,7 @@ generate_key_expansion (struct key_ctx_bi *key,
   /* compute key expansion */
   openvpn_PRF (master,
 	       sizeof(master),
-	       "OpenVPN key expansion",
+	       PACKAGE_NAME " key expansion",
 	       key_src->client.random2,
 	       sizeof(key_src->client.random2),
 	       key_src->server.random2,
@@ -1878,7 +1883,7 @@ tls_process (struct tls_multi *multi,
 	  if (!to_link->len && !reliable_ack_empty (&ks->rec_ack))
 	    {
 	      buf = &ks->ack_write_buf;
-	      ASSERT (buf_init (buf, EXTRA_FRAME (&multi->opt.frame)));
+	      ASSERT (buf_init (buf, FRAME_HEADROOM (&multi->opt.frame)));
 	      write_control_auth (session, ks, buf, to_link_addr, P_ACK_V1,
 				  RELIABLE_ACK_SIZE, false, current);
 	      *to_link = *buf;
@@ -2173,7 +2178,7 @@ tls_process (struct tls_multi *multi,
   if (!to_link->len && !reliable_ack_empty (&ks->rec_ack))
     {
       buf = &ks->ack_write_buf;
-      ASSERT (buf_init (buf, EXTRA_FRAME (&multi->opt.frame)));
+      ASSERT (buf_init (buf, FRAME_HEADROOM (&multi->opt.frame)));
       write_control_auth (session, ks, buf, to_link_addr, P_ACK_V1,
 			  RELIABLE_ACK_SIZE, false, current);
       *to_link = *buf;
@@ -2673,7 +2678,7 @@ tls_pre_decrypt (struct tls_multi *multi,
 	    }
 
 	  msg (D_TLS_ERRORS,
-	       "TLS Error: Unknown data channel key ID or IP address received from %s: %d",
+	       "TLS Error: Unknown data channel key ID or IP address received from %s: %d (see FAQ for more info on this error)",
 	       print_sockaddr (from), key_id);
 	  goto error;
 	}
